@@ -31,7 +31,7 @@
 
 	virtual void SetCmdStr(char* str);
 	virtual void SetCmdVal(ItemVal val, ItemSetType st);
-	virtual void CmdCallback(int id, UINT codeNotify);
+	virtual void CmdCallback(int id, WORD codeNotify);
 
 	virtual int MEASUREITEM(int id, MEASUREITEMSTRUCT* mis);
 	virtual int DRAWITEM(int id, DRAWITEMSTRUCT* dis);
