@@ -41,6 +41,6 @@
 	int _minVal;		// Can scale to min and val
 	int _maxVal;
 
-	virtual void CmdCallback(int id, UINT codeNotify);
+	virtual void CmdCallback(int id, WORD codeNotify);
   };
 #endif
