@@ -34,9 +34,7 @@
       protected:	//--------------------------------------- protected
 
 	virtual int GetPaneValue(ItemVal id, int& val);
-	virtual void* GetPaneStruct(ItemVal id, int getType);
 	virtual void SetPaneValue(ItemVal id, int val, ItemSetType setType);
-	virtual void SetPaneStruct(ItemVal id, void* setTo, int setType);
 	virtual void SetPaneString(ItemVal id, char* str);
 
 
