@@ -35,7 +35,7 @@
 //                                                                         //
 //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++//
 //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++//
-
+#include <v/vwin32.h>		// for Win 32 stuff
 #include <v/vapp.h>		// my header file
 #include <v/vwindow.h>		// Win header
 #include <v/vfont.h>		// for font stuff
@@ -45,6 +45,10 @@
 
 #include <stdlib.h>
 
+#ifdef _WIN32
+#define _export
+#endif
+ 
     // Globals available to the world
 
     vApp* theApp = NULL;	// to be filled in upon instantiation
@@ -63,7 +67,7 @@
  GNU Library General Public License, version 2 <****";
 
 //========================>>> vApp::vApp <<<=======================
-  vApp::vApp(char* appName) : vBaseItem(appName)	// constructor
+  vApp::vApp(char* appName, int simSDI) : vBaseItem(appName)	// constructor
   {
     // First, set the global pointer to the main App. This happens
     // when the user declares the instance of the app, either from
@@ -79,11 +83,14 @@
     _WindowList = NULL;		// no windows registered
     _CmdPaneList = NULL;	// no command panes registered
     _inExit = 0;
+    _simSDI = simSDI;
 
     _DefaultHeight = 300;	// default sizes for canvas window
     _DefaultWidth = 500;
     _destroyed = 0;		// Not destroyed yet
 
+    _Frame = 0;	                // Not inited yet.
+
     // Set which debug items to show
 
     DebugState.System = 0;			// System debug messages
@@ -179,8 +186,8 @@
 	      }
 	    
 	    // Now fixup the argument list and break loop
-
-	    for (int ia = argn ; ia < argc ; ++ia)
+	    int ia;
+	    for (ia = argn ; ia < argc ; ++ia)
 		argv[ia] = argv[ia+2];
 	    argv[ia] = 0;
 	    argc -= 2;	// eat the -vDebug args
@@ -200,21 +207,21 @@
       {
 	// Register the MDI frame window class
 
-	_appicon = ::LoadIcon(hInstance,"vAppIcon");
+	_appicon = ::LoadIcon((HINSTANCE)hInstance,"vAppIcon");
 	if (_appicon == NULL)
 	    _appicon = ::LoadIcon(NULL,IDI_APPLICATION);
-	_winicon = ::LoadIcon(hInstance,"vWindowIcon");
+	_winicon = ::LoadIcon((HINSTANCE)hInstance,"vWindowIcon");
 	if (_winicon == NULL)
 	    _winicon = ::LoadIcon(NULL,IDI_APPLICATION);
 
 	wndclass.style         = CS_HREDRAW | CS_VREDRAW;
-	wndclass.lpfnWndProc   = PMdiFrameProc;
+	wndclass.lpfnWndProc   = (WNDPROC) PMdiFrameProc;
 	wndclass.cbClsExtra    = 0;
 	wndclass.cbWndExtra    = 0;
-	wndclass.hInstance     = hInstance;
+	wndclass.hInstance     = (HINSTANCE) hInstance;
 	wndclass.hIcon         = _appicon;
 	wndclass.hCursor       = LoadCursor (NULL, IDC_ARROW);
-	wndclass.hbrBackground = COLOR_APPWORKSPACE + 1;
+	wndclass.hbrBackground = (HBRUSH)(COLOR_APPWORKSPACE + 1);
 	wndclass.lpszMenuName  = NULL;
 	wndclass.lpszClassName = szMdiFrameClass;
 
@@ -223,13 +230,13 @@
 	// Register the MDI window class
 
 	wndclass.style         = CS_HREDRAW | CS_VREDRAW ;
-	wndclass.lpfnWndProc   = PMdiWindowProc;
+	wndclass.lpfnWndProc   = (WNDPROC) PMdiWindowProc;
 	wndclass.cbClsExtra    = 0;
 	wndclass.cbWndExtra    = sizeof(void*);
-	wndclass.hInstance     = hInstance;
+	wndclass.hInstance     = (HINSTANCE) hInstance;
 	wndclass.hIcon         = _winicon;
 	wndclass.hCursor       = NULL; //LoadCursor (NULL, IDC_ARROW);
-	wndclass.hbrBackground = ::GetStockObject(WHITE_BRUSH);
+	wndclass.hbrBackground = (HBRUSH)::GetStockObject(WHITE_BRUSH);
 	wndclass.lpszMenuName  = NULL;
 	wndclass.lpszClassName = szMdiWindowClass;
 
@@ -242,11 +249,9 @@
 
     // Define a very simple default MDI menu
 
-
-  //  ::AppendMenu(hFrameSubMenu, MF_STRING, M_New, "&New");
     ::AppendMenu(hFrameSubMenu, MF_STRING, M_WindowsReserved1, "E&xit");
 
-    ::AppendMenu(hFrameMenu, MF_POPUP, hFrameSubMenu, "&File");
+    ::AppendMenu(hFrameMenu, MF_POPUP, (UINT)hFrameSubMenu, "&File");
 
     hAccel = 0;				// No accelerators
 
@@ -257,7 +262,7 @@
 	WS_VISIBLE | WS_MAXIMIZEBOX | WS_MINIMIZEBOX,
 	CW_USEDEFAULT, CW_USEDEFAULT,
 	CW_USEDEFAULT, CW_USEDEFAULT,
-	NULL, hFrameMenu, hInstance, NULL);
+	NULL, hFrameMenu, (HINSTANCE)hInstance, NULL);
 
     _Client = ::GetWindow(_Frame, GW_CHILD);
 
@@ -274,7 +279,7 @@
     _BarBrush = ::CreateSolidBrush(::GetSysColor(COLOR_BTNFACE));
 
     ::ShowWindow(_Frame, nCmdShow);
-  
+
     ::UpdateWindow(_Frame);
 
     _running = 1;
@@ -311,6 +316,7 @@
 	    CloseAppWin(tmp);		// use local or derived close app
       }
 
+    _inExit = 0;			// done now, so can exit
     AppExit(0);
   }
 
@@ -318,7 +324,33 @@
   void vApp::AppExit(int exitVal)
   {
 
-   ::PostQuitMessage(exitVal);
+    if (!_inExit)		// Only ONE, please!
+      {
+	::PostQuitMessage(exitVal);
+      }
+  }
+
+//========================>>> vApp::CheckEvents <<<=======================
+  void vApp::CheckEvents()
+  {
+    extern int vChkWinDlgMsgs(MSG*);
+    extern int vChkCmdPaneMsgs(MSG*);
+    MSG msg;
+
+    // Enter the modified message loop
+
+    while (::PeekMessage(&msg, NULL, 0, 0, PM_NOYIELD | PM_REMOVE))
+      {
+
+	if (!vChkWinDlgMsgs(&msg) &&		// Dialogs and cmd panes handle
+	  !vChkCmdPaneMsgs(&msg) &&             // their own messages
+	  !::TranslateMDISysAccel(_Client, &msg) &&
+	  !::TranslateAccelerator(_Frame, (HACCEL)hAccel, &msg)) // probably don't need?
+	  {
+	    ::TranslateMessage(&msg);
+	    ::DispatchMessage(&msg);
+	  }
+      }
   }
 
 //========================>>> vApp::CloseAppWin <<<=======================
@@ -328,6 +360,7 @@
     SysDebug(Build,"vApp::CloseAppWin()\n");
 
     win->CloseWin();			// let the window close itself
+  
     unregisterWindow(win);		// take it off the list
     delete win;				// free the window
   }
@@ -370,7 +403,7 @@
   vFont vApp::GetDefaultFont()
   {
     vFont sysF(vfDefaultSystem);	// construct a system font instance
-    return sysF; 			// return it
+    return sysF; 		      	// return it
   }
 
 //========================>>> vApp::getAppWinInfo <<<=======================
@@ -448,17 +481,17 @@
 //========================>>> vApp::unregisterWindow <<<=======================
   void vApp::unregisterWindow(vWindow *Win)
   {
-    // Scan window list to unregister this window and free some space
+    // Scan window list to unregister this window and free some space 
 
-    WindList *curWin, *tmp, *last;
+    WindList *curWin, *tmp, *last, *next;
 
     last = 0;
-
-    for (curWin = _WindowList ; curWin !=0 ; curWin = curWin->nextWinList)
+   
+    for (curWin = _WindowList ; curWin !=0 ; curWin = next)
       {
+	next = curWin->nextWinList;
 	if (curWin->window == Win)
 	  {
-
 	    SysDebug1(Misc,"vApp::unregisterWindow - %s\n",Win->name())
 
 	    tmp = curWin;
@@ -492,12 +525,13 @@
   void vApp::unregisterCmdPane(vCommandPane* cmdPane)
   {
     // Scan pane list to unregister this window and free some space
-    CmdPaneList *curCP, *tmp, *last;
+    CmdPaneList *curCP, *tmp, *last, *next;
 
     last = 0;
 
-    for (curCP = _CmdPaneList ; curCP !=0 ; curCP = curCP->nextCPList)
+    for (curCP = _CmdPaneList ; curCP !=0 ; curCP = next)
       {
+	next = curCP->nextCPList;
 	if (curCP->commandPane == cmdPane)
 	  {
 
@@ -525,6 +559,8 @@
     // and turn onn all cmd panes that are in our window
     CmdPaneList *curCP;
 
+    // First, turn off all command panes in other windows
+
     for (curCP = _CmdPaneList ; curCP !=0 ; curCP = curCP->nextCPList)
       {
 	if ((curCP->commandPane)->_parentWin != parent)
@@ -534,7 +570,12 @@
 	       ::IsWindowVisible((curCP->commandPane)->_wDialog))
 		   ::ShowWindow((curCP->commandPane)->_wDialog, SW_HIDE);
 	  }
-	else
+      }
+
+    // Now, turn ours on
+    for (curCP = _CmdPaneList ; curCP !=0 ; curCP = curCP->nextCPList)
+      {
+	if ((curCP->commandPane)->_parentWin == parent)
 	  {
 	    (curCP->commandPane)->_isShown = 1;
 	    if (::IsWindow((curCP->commandPane)->_wDialog) &&
@@ -542,6 +583,7 @@
 		   ::ShowWindow((curCP->commandPane)->_wDialog, SW_SHOW);
 	  }
       }
+  // FORWARD_vWM_ResizeMDIClient(theApp->_Frame,SendMessage);
   }
 
 //========================>>> vApp::SendWindowCommandAll <<<=======================
@@ -608,13 +650,13 @@
 
     // Enter the modified message loop
 
-    while (GetMessage(&msg, NULL, 0, 0))
+    while (::GetMessage(&msg, NULL, 0, 0))
       {
 
 	if (!vChkWinDlgMsgs(&msg) &&		// Dialogs and cmd panes handle
 	  !vChkCmdPaneMsgs(&msg) &&             // their own messages
 	  !::TranslateMDISysAccel(_Client, &msg) &&
-	  !::TranslateAccelerator(_Frame, hAccel, &msg)) // probably don't need?
+	  !::TranslateAccelerator(_Frame, (HACCEL)hAccel, &msg)) // probably don't need?
 	  {
 	    ::TranslateMessage(&msg);
 	    ::DispatchMessage(&msg);
@@ -649,7 +691,7 @@
 
 //======================>>> PMdiFrameProc <<<================================
   long FAR PASCAL _export PMdiFrameProc(HWND hwnd, UINT message,
-				UINT wParam, LONG lParam)
+				UINT wParam, LPARAM lParam)
   {
     return theApp->MdiFrameProc(hwnd, message, wParam, lParam);
   }
@@ -663,11 +705,22 @@
     clientcreate.hWindowMenu  = hFrameSubMenu;
     clientcreate.idFirstChild = M_FIRSTCHILD;
 
-    _Client = ::CreateWindow("MDICLIENT", NULL,
-	WS_CHILD | WS_CLIPCHILDREN | WS_VISIBLE | WS_CLIPSIBLINGS |
-	WS_HSCROLL | WS_VSCROLL,
-	0, 0, 0, 0, hwnd, NULL, _vHandle,
-	(LPSTR) &clientcreate);
+    if (_simSDI)
+      {
+        _Client = ::CreateWindow("MDICLIENT", NULL,
+	  WS_CHILD | WS_CLIPCHILDREN | WS_VISIBLE | WS_CLIPSIBLINGS |
+	  WS_HSCROLL | WS_VSCROLL | MDIS_ALLCHILDSTYLES,
+	  0, 0, 0, 0, hwnd, NULL, (HINSTANCE)_vHandle,
+	  (LPSTR) &clientcreate);
+      }
+    else
+      {
+	_Client = ::CreateWindow("MDICLIENT", NULL,
+	  WS_CHILD | WS_CLIPCHILDREN | WS_VISIBLE | WS_CLIPSIBLINGS |
+	  WS_HSCROLL | WS_VSCROLL,
+	  0, 0, 0, 0, hwnd, NULL, (HINSTANCE)_vHandle,
+	  (LPSTR) &clientcreate);
+      }
 
     return _Client != NULL;
   }
@@ -718,13 +771,6 @@
 	    FORWARD_WM_CLOSE(hwnd, SendMessage);
 	    return;
 
- //	case M_New:
- //	  {
- //	    vCmdWindow* t = new vCmdWindow("First Window", 100, 400);
- //	    theApp->NewAppWin(t, "First Window",100,400);
- //         return;
- //	  }
-
 	// Messages for arranging child MDI windows
 	case M_Tile:
 	    (void)FORWARD_WM_MDITILE(_Client, WM_MDITILE, SendMessage);
@@ -740,12 +786,15 @@
 
 	default:            // Pass to active child
 
-	    HWND hwndChild = LOWORD(::SendMessage(_Client,
-		WM_MDIGETACTIVE, 0, 0L));
+	    HWND hwndChild = (HWND)::SendMessage(_Client,
+		WM_MDIGETACTIVE, 0, 0L);
 
-	    if (::IsWindow(hwndChild))
-	      FORWARD_WM_COMMAND(hwndChild, id, hwndCtl, codeNotify,
-      		    SendMessage);
+	    if (::IsWindow(hwndChild) && id < M_FIRSTCHILD)
+	      {
+	        FORWARD_WM_COMMAND(hwndChild, id, hwndCtl, codeNotify,
+		    SendMessage);
+		return;
+	      }
       }
     FORWARD_WM_COMMAND(hwnd, id, hwndCtl, codeNotify, Frame_DefProc);
   }
@@ -770,10 +819,10 @@
     if (!::IsWindow(_Client))
 	return;
 
-    ::GetClientRect(_Frame, &rc);		 // the rect of the Frame
+   if (!_Frame)
+      return;
 
-    if (rc.top != 0)	// Windows generates this message even when the
-	return;		// frame doesn't have a valid rect yet -- ignore...
+    ::GetClientRect(_Frame, &rc);		 // the rect of the Frame
 
     int topY = 0;			// Need to track space used by
     int botY = rc.bottom;               // cmd bars and status bars
@@ -807,7 +856,7 @@
   }
 
 //======================>>> vApp::MdiFrameProc <<<============================
-  long vApp::MdiFrameProc(HWND hwnd, UINT uMsg, UINT wParam, LONG lParam)
+  long vApp::MdiFrameProc(HWND hwnd, UINT uMsg, UINT wParam, LPARAM lParam)
   {
     switch (uMsg)
      {
@@ -863,8 +912,13 @@
 //#########################################################################
 
 //======================>>> WinMain <<<======================================
+#ifdef _WIN32
   int PASCAL WinMain(HANDLE hInstance, HANDLE hPrevInstance,
 		      LPSTR lpszCmdLine, int nCmdShow)
+#else
+  int PASCAL WinMain(HANDLE hInstance, HANDLE hPrevInstance,
+		      LPSTR lpszCmdLine, int nCmdShow)
+#endif
   {
    return CMain(hInstance, hPrevInstance, lpszCmdLine, nCmdShow);
   }
@@ -890,7 +944,7 @@
 
     // Get application name
 
-    ::GetModuleFileName(hInstance, name, 199);
+    ::GetModuleFileName((HINSTANCE)hInstance, name, 199);
 
     argv[argc++] = name;
 
