@@ -10,12 +10,14 @@
 //===============================================================
 
 
+#include <v/vwin32.h>		// for Win 32 stuff
 extern "C"
 {
 #include <math.h>
 }
 
 #include <v/vwindc.h>
+#include <v/vmemdc.h>
 
 #include <v/vapp.h>		// need access to the app
 #include <v/vcanvas.h>		// our own canvas widget
@@ -31,7 +33,8 @@ extern "C"
     _font = theApp->GetDefaultFont();	// Use the default fixed font by default!
 
     _hdc = 0;  _hdcPaint = 0;
-    _hpen = ::GetStockObject(BLACK_PEN); _hbrush = ::GetStockObject(WHITE_BRUSH);
+    _hpen = (HPEN) ::GetStockObject(BLACK_PEN);
+    _hbrush = (HBRUSH) ::GetStockObject(WHITE_BRUSH);
     _canvasBG = RGB(255,255,255);
     _isPrinterDC = 0;			// Assume not printer
   }
@@ -44,6 +47,38 @@ extern "C"
 
   }
 
+//=====================>>> vXDC::CopyFromMemoryDC <<<==========================
+  void vWinDC::CopyFromMemoryDC(vMemoryDC* vmemdc, int destX, int destY)
+  {
+    // Copy to myself from another DC
+    int ch = vmemdc->_physHeight;
+    int cw = vmemdc->_physWidth;
+    register COLORREF color, cout;
+
+    if (cw > _physWidth && _physWidth > 0)	// only copy what will fit
+	cw = _physWidth;
+    if (ch > _physHeight && _physHeight > 0)
+	ch = _physHeight;
+
+    GetHDC();			// Get my hdc
+    vmemdc->GetHDC();		// the source hdc
+    HDC memDC = vmemdc->_hdc;
+
+#define BITBLT
+#ifdef BITBLT
+    ::BitBlt(_hdc, 0, 0, cw, ch, memDC, 0, 0, SRCCOPY);
+#else
+
+    for (int row = 0 ; row < ch ; ++row)
+	for (int col = 0 ; col < cw ; ++col)
+	  {
+	     color = ::GetPixel(memDC,col,row);
+	     cout = ::SetPixel(_hdc,col,row,color);
+	  }
+#endif
+     vmemdc->ReleaseHDC();
+     ReleaseHDC();
+   }
 
 #ifdef DRAWARC
 //====================>>> vWinDC::DrawArc <<<==========================
@@ -144,8 +179,6 @@ const double PIx2 = 6.283185;
     int xx = Scale(x+_tx);
     int yy = Scale(y+_ty);
 
-    icon.MapIcon();		// need to map to internal format
-
     // Now, draw the icon
     GetHDC();			// get our hdc
     ::SetBkColor(_hdc, _canvasBG);
@@ -156,7 +189,7 @@ const double PIx2 = 6.283185;
 
     if (_isPrinterDC)
       {
-	hbrush = ::GetStockObject(WHITE_BRUSH);
+	hbrush = (HBRUSH) ::GetStockObject(WHITE_BRUSH);
 	::SelectObject(_hdc, ::GetStockObject(BLACK_PEN));
       }
     else
@@ -164,12 +197,21 @@ const double PIx2 = 6.283185;
 
     ::SelectObject(_hdc, hbrush);
 
-    ::SelectObject(hdcMem,icon._hbm);
+    HBITMAP hbm = icon.GetIconHBM(_hdc);	// get the bitmap
+    ::SelectObject(hdcMem,hbm);
 
     // Now BitBlt the icon BMP into the drawing canvas, leaving
     // background untouched, and the pen color to draw 1 bits
-    ::BitBlt(_hdc, xx, yy, icon.width, icon.height,
-	hdcMem, 0, 0, 0xE20746L);
+    if (icon.depth == 1)
+      {
+	::BitBlt(_hdc, xx, yy, icon.width, icon.height,
+	  hdcMem, 0, 0, 0xE20746L);
+      }
+    else
+      {
+	::BitBlt(_hdc, xx, yy, icon.width, icon.height,
+	    hdcMem, 0, 0, SRCCOPY);
+      }
     ::DeleteDC(hdcMem);
     if (!_isPrinterDC)
 	::DeleteObject(hbrush);	// free brush
@@ -230,6 +272,83 @@ const double PIx2 = 6.283185;
     ReleaseHDC();			// Release the DC
   }
 
+//====================>>> vWinDC::DrawLines <<<==========================
+  void vWinDC::DrawLines(vLine* lineList)
+  {
+
+
+    GetHDC();				// Get a DC to use
+    _hpen = _pen.GetHPEN();		// Get the pen
+    ::SelectObject(_hdc, _hpen);    	// Select it
+
+
+    for (vLine* lp = lineList ; lp ; ++lp)
+      {
+
+	int xx = Scale(lp->x+_tx);
+	int yy = Scale(lp->y+_ty);
+	int xe = Scale(lp->xend+_tx);
+	int ye = Scale(lp->yend+_ty);
+
+	::MoveTo(_hdc,xx, yy);
+	::LineTo(_hdc, xe, ye);
+      }
+
+    ::SelectObject(_hdc, ::GetStockObject(BLACK_PEN));	// something safe
+
+    ReleaseHDC();			// Release the DC
+  }
+
+//==================>>> vWinDC::DrawColorPoints <<<======================
+  void vWinDC::DrawColorPoints(int x, int y, int nPoints, vColor* pointList)
+  {
+    // Draw a bunch of color points. 
+
+    if (nPoints < 1)
+	return;
+
+    int xx = Scale(x+_tx);
+    int yy = Scale(y+_ty);
+
+    if (yy < 0)
+	return;				// row not displayed
+
+    GetHDC();				// Get a DC to use
+
+    COLORREF curpix = pointList[0].pixel();
+    COLORREF prevpix = curpix;
+    COLORREF usepix = curpix;		// rep used by windows
+
+    if (xx < 0)				// need to check every time
+      {
+	for (int ix = 0 ; ix < nPoints ; ++ix)
+	  {
+	    if (xx+ix < 0)
+		continue;			// this one not displayed
+	    curpix = pointList[ix].pixel();
+	    if (curpix != prevpix)		// don't reset color if same
+	      {
+		usepix = prevpix = curpix;
+	      }
+	    usepix = ::SetPixel(_hdc,xx+ix,yy,usepix);
+	  }
+      }
+    else				// don't need xx check
+      {
+	for (int ix = 0 ; ix < nPoints ; ++ix)
+	  {
+	    curpix = pointList[ix].pixel();
+	    if (curpix != prevpix)		// don't reset color if same
+	      {
+		usepix = prevpix = curpix;
+	      }
+	    usepix = ::SetPixel(_hdc,xx+ix,yy,usepix);
+	  }
+      }
+
+    ReleaseHDC();			// Release the DC
+  }
+
 //==================>>> vWinDC::DrawPoint <<<======================
   void vWinDC::DrawPoint(int x, int y)
   {
@@ -254,6 +373,33 @@ const double PIx2 = 6.283185;
 
   }
 
+//==================>>> vWinDC::DrawPoints <<<======================
+  void vWinDC::DrawPoints(vPoint* pointList)
+  {
+
+    // First, do the required scaling
+
+    GetHDC();				// Get a DC to use
+    _hpen = _pen.GetHPEN();		// Get the pen
+    ::SelectObject(_hdc, _hpen);    	// Select it
+
+       // Now the code to actually do the drawing
+
+    for (vPoint* pl = pointList ; pl ; ++pl)
+      {
+	int xx = Scale(pl->x+_tx);
+	int yy = Scale(pl->y+_ty);
+
+	::MoveTo(_hdc,xx, yy);
+	::LineTo(_hdc, xx+1, yy+1);
+      }
+
+    ::SelectObject(_hdc, ::GetStockObject(BLACK_PEN));	// something safe
+
+    ReleaseHDC();			// Release the DC
+
+  }
+
 //==================>>> vWinDC::DrawPolygon <<<======================
   void vWinDC::DrawPolygon(int n, vPoint points[], int fillStyle)
   {
@@ -272,9 +418,20 @@ const double PIx2 = 6.283185;
 	  }
       }
     else
+#ifdef _WIN32
+      {
+	xpoints = new POINT[n];	// allocate space for copy
+	for (int i = 0; i < n; i++)
+	  {
+	    xpoints[i].x = points[i].x;
+	    xpoints[i].y = points[i].y;
+	  }
+      }
+#else
       {
 	xpoints = (POINT*)points;	// just use original values
       }
+#endif
 
     GetHDC();				// Get a DC to use
     _hpen = _pen.GetHPEN();		// Get the pen
@@ -293,7 +450,9 @@ const double PIx2 = 6.283185;
 
     ReleaseHDC();			// Release the DC
 
+#ifndef _WIN32
     if (_hasScale)
+#endif
 	delete[]xpoints;
 }
 
@@ -305,10 +464,17 @@ const double PIx2 = 6.283185;
 
     int xx = Scale(x+_tx);
     int yy = Scale(y+_ty);
-    int x2 = Scale(x+width+_tx);
-    int y2 = Scale(y+height+_ty);
+    int x2 = Scale(x+width+1+_tx);
+    int y2 = Scale(y+height+1+_ty);
 
     GetHDC();				// Get a DC to use
+    if (height == 1 && width == 1)
+      {
+	COLORREF curpix = _brush.GetColor().pixel();
+	::SetPixel(_hdc,xx,yy,curpix);
+	ReleaseHDC();
+	return;
+      }
     _hpen = _pen.GetHPEN();		// Get the pen
     ::SelectObject(_hdc, _hpen);    	// Select it
     _hbrush = _brush.GetHBRUSH();
@@ -323,6 +489,43 @@ const double PIx2 = 6.283185;
 
   }
 
+//==================>>> vWinDC::DrawRectangles <<<======================
+  void vWinDC::DrawRectangles(vRect* rectList)
+  {
+    GetHDC();				// Get a DC to use
+
+    _hpen = _pen.GetHPEN();		// Get the pen
+    ::SelectObject(_hdc, _hpen);    	// Select it
+    _hbrush = _brush.GetHBRUSH();
+    ::SelectObject(_hdc, _hbrush);
+
+    for (vRect* rp = rectList ; rp ; ++rp)
+      {
+	int xx = Scale(rp->x+_tx);
+	int yy = Scale(rp->y+_ty);
+	int x2 = Scale(rp->x+rp->w+1+_tx);
+	int y2 = Scale(rp->y+rp->h+1+_ty);
+
+	if (rp->h == 0 && rp->w == 0)
+	    continue;
+
+	if (rp->h == 1 && rp->w == 1)
+	  {
+	    COLORREF curpix = _brush.GetColor().pixel();
+	    ::SetPixel(_hdc,xx,yy,curpix);
+	  }
+
+	else
+	    (void)::Rectangle(_hdc, xx, yy, x2, y2);
+      }
+
+    ::SelectObject(_hdc, ::GetStockObject(BLACK_PEN));	// something safe
+    ::SelectObject(_hdc, ::GetStockObject(WHITE_BRUSH));	// something safe
+
+    ReleaseHDC();			// Release the DC
+
+  }
+
 //================>>> vWinDC::DrawRoundedRectangle <<<===================
   void vWinDC::DrawRoundedRectangle(int x, int y,
 	int width, int height, int radius)
@@ -529,6 +732,9 @@ const double PIx2 = 6.283185;
 
     // First, do the required scaling
 
+    if (!text || !*text)
+	return;
+
     int xx = Scale(x+_tx);
     int yy = Scale(y+_ty);
 
@@ -608,10 +814,15 @@ const double PIx2 = 6.283185;
     _font.LoadFont(_hdc,_isPrinterDC);		// need to setup font!
     ::SelectObject(_hdc, _font.GetHFONT());	// select it in
 
+#ifdef _WIN32
+    SIZE size;
+    :: GetTextExtentPoint32 (_hdc, str, len,&size);
+    ReleaseHDC();			// Release the DC 
+    return size.cx;
+#else
     DWORD extent = ::GetTextExtent(_hdc, str, len);
-
     ReleaseHDC();			// Release the DC 
-
     // call x to get the length
     return LOWORD(extent);
+#endif
   }
