@@ -9,9 +9,11 @@
 // vapp.cxx for more complete information about license terms.
 //===============================================================
 
+#include <v/vwin32.h>		// for Win 32 stuff
 #include <v/vapp.h>		// we are a friend of App
 #include <v/vwindow.h>		// our header
 #include <v/vpane.h>		// we have panes
+#include <v/vmenu.h>
 #include <v/vkeys.h>
 
 #include <v/vnotice.h>
@@ -50,6 +52,7 @@
     _mouseDown = 0;
     _shiftKeyDown = 0;
     _ctrlKeyDown = 0;
+    _createdOK = 0;
 
     initialize();		// set this guy up
   }
@@ -57,7 +60,7 @@
   vWindow::vWindow(const vWindow& w) : 		// Copy Constructor
     vBaseWindow(w)
   {
-    // Copy is shallow, but we track that it is a copy
+    // Copy is shallow, but we track that it is a copy                     
     vSysError("vWindow - V semantics do not support copy constructors!");
   }
 
@@ -65,7 +68,7 @@
   vWindow::~vWindow()			// destructor
   {
     SysDebug(Destructor,"vWindow::~vWindow destructor\n")
-
+  
     PaneList* temp;
     for (PaneList* pl = _paneList ; pl != 0 ; )
       {
@@ -75,7 +78,7 @@
       }
     if (_menuBar != 0)		// clean up the menu bar
       {
-	::DestroyMenu(_menuBar);
+ 	::DestroyMenu(_menuBar);
 	_menuBar = 0;
       }
   }
@@ -96,29 +99,41 @@
 
     if (add_pane->_paneType == P_Menu)	// P_Menu is special
       {
+	_menuPane = add_pane;
 	add_pane->initialize(this, (HWND)_menuBar);
 	HMENU popup = ::GetSubMenu(_menuBar,_WindowMenuIndex); // get 1st menu item
 
-	if (popup)			// add the window list
-	    ::SendMessage(theApp->_Client, WM_MDISETMENU,0,
-			MAKELONG(_menuBar,popup));
-	::DrawMenuBar(theApp->_Frame);	// Make it change
+	if (popup)		// add the window list
+	  {
+#ifdef _WIN32
+	    BOOL refresh = 1;
+#else
+	    BOOL refresh = 0;
+#endif
+	    FORWARD_WM_MDISETMENU(theApp->_Client, refresh,
+		(HMENU) _menuBar, popup, ::SendMessage);
+	  }
+	
+	::DrawMenuBar(theApp->_Frame); // Make it change
       }
     else
-	add_pane->initialize(this, _myHwnd);	// add to _toplevel paneWidget
+      add_pane->initialize(this, _myHwnd); // add to _toplevel paneWidget
   }
 
 //======================>>> vWindow::initialize <<<=======================
   void vWindow::initialize(void)
   {
-    // All top level windows in this frame are implemented
-    // as MDI child windows
+  // All top level windows in this frame are implemented
+  // as MDI child windows
 
+    if (_numWindows > 0 && theApp->simSDI())
+	return;
+  
     MDICREATESTRUCT mdic;
+  
+    _menuBar = ::CreateMenu(); // Menu Bar
 
-    _menuBar = ::CreateMenu();	// Menu Bar
-
-    mdic.szClass = "MdiWindowClass";	// Standard Windows setup
+    mdic.szClass = "MdiWindowClass"; // Standard Windows setup
     mdic.szTitle = _name;
     mdic.hOwner = theApp->vHandle();
     mdic.x = mdic.y = CW_USEDEFAULT;
@@ -129,93 +144,99 @@
 
     mdic.cx = _WinWidth+(2*fx);
     mdic.cy = _WinHeight + (2*fy) + titley;
-    mdic.style = 0;
-    mdic.lParam = (LPARAM)this;	  	// send this to PMdiWindowProc
+    if (theApp->simSDI())
+	mdic.style = WS_MAXIMIZE | WS_CHILD | WS_CLIPSIBLINGS
+	  | WS_CLIPCHILDREN | WS_VISIBLE;
+    else
+    	mdic.style = 0;
+    mdic.lParam = (LPARAM)this; // send this to PMdiWindowProc
 
     _myHwnd = (HWND)::SendMessage(theApp->_Client,
-	    WM_MDICREATE,0, (long)(LPMDICREATESTRUCT) &mdic);
-    // save a copy of this in the window structure
+	WM_MDICREATE,0, (long)(LPMDICREATESTRUCT) &mdic);
 
+    // save a copy of this in the window structure
+  
     ::SetWindowLong(_myHwnd, 0 , (long)this);
 
     ++_numWindows;		// bump how many windows up
     _cursor = ::LoadCursor(NULL, IDC_ARROW);
     ::SetFocus(_myHwnd);
+    _createdOK = 1;
   }
 
 //============================>>> vWindow::SetWinCursor <<<==================
   void vWindow::SetWinCursor(VCursor vcursor)
   {
-
+  
     switch (vcursor)
       {
 	case VC_None:
-	  _cursor = ::LoadCursor(NULL, IDC_ARROW);
-	  break;
+	    _cursor = ::LoadCursor(NULL, IDC_ARROW);
+	    break;
 	case VC_Arrow:
-	  _cursor = ::LoadCursor(NULL, IDC_ARROW);
-	  break;
+	    _cursor = ::LoadCursor(NULL, IDC_ARROW);
+	    break;
 	case VC_CenterArrow:
-	  _cursor = ::LoadCursor(NULL, IDC_UPARROW);
-	  break;
+	    _cursor = ::LoadCursor(NULL, IDC_UPARROW);
+	    break;
 	case VC_CrossHair:
-	  _cursor = ::LoadCursor(NULL, IDC_CROSS);
-	  break;
+	    _cursor = ::LoadCursor(NULL, IDC_CROSS);
+	    break;
 	case VC_EWArrows:
-	  _cursor = ::LoadCursor(NULL, IDC_SIZEWE);
-	  break;
+	    _cursor = ::LoadCursor(NULL, IDC_SIZEWE);
+	    break;
 	case VC_Hand:
-	  _cursor = ::LoadCursor(NULL, IDC_ARROW);
-	  break;
+	    _cursor = ::LoadCursor(NULL, IDC_ARROW);
+	    break;
 	case VC_IBar:
-	  _cursor = ::LoadCursor(NULL, IDC_IBEAM);
-	  break;
+	    _cursor = ::LoadCursor(NULL, IDC_IBEAM);
+	    break;
 	case VC_Icon:
-	  _cursor = ::LoadCursor(NULL, IDC_ICON);
-	  break;
+	    _cursor = ::LoadCursor(NULL, IDC_ICON);
+	    break;
 	case VC_NSArrows:
-	  _cursor = ::LoadCursor(NULL, IDC_SIZENS);
-	  break;
+	    _cursor = ::LoadCursor(NULL, IDC_SIZENS);
+	    break;
 	case VC_Pencil:
-	  _cursor = ::LoadCursor(NULL, IDC_ARROW);
-	  break;
+	    _cursor = ::LoadCursor(NULL, IDC_ARROW);
+	    break;
 	case VC_Question:
-	  _cursor = ::LoadCursor(NULL, IDC_CROSS);
-	  break;
+	    _cursor = ::LoadCursor(NULL, IDC_CROSS);
+	    break;
 	case VC_Sizer:
-	  _cursor = ::LoadCursor(NULL, IDC_SIZE);
-	  break;
+	    _cursor = ::LoadCursor(NULL, IDC_SIZE);
+	    break;
 	case VC_Wait:
-	  _cursor = ::LoadCursor(NULL, IDC_WAIT);
-	  break;
+	    _cursor = ::LoadCursor(NULL, IDC_WAIT);
+	    break;
 	case VC_X:
-	  _cursor = ::LoadCursor(NULL, IDC_ARROW);
-	  break;
+	    _cursor = ::LoadCursor(NULL, IDC_ARROW);
+	    break;
 	default:
-	  _cursor = ::LoadCursor(NULL, IDC_ARROW);
-	  break;
+	    _cursor = ::LoadCursor(NULL, IDC_ARROW);
+	    break;
       }
   }
 
 //============================>>> vWindow::CloseWin <<<==========================
   void vWindow::CloseWin(void)
   {
-    //	close button class callback
-    //
-    vWindow* w_list;
-
+  //	close button class callback
+  //
+  vWindow* w_list;
+  
     // close base window, which cleans up some stuff
-
+  
     SysDebug1(Build,"vWindow::CloseWin() - %s.\n",_name)
 
     closeBaseWindow();		// close this window
-
+  
     --_numWindows;
     // Remove from active list
 
-    if (_WinList == this)		// first one on list
+    if (_WinList == this)	// first one on list
       {
-	_WinList = _WinList->_NextWin;	// point to next
+	_WinList = _WinList->_NextWin; // point to next
       }
     else
       {
@@ -229,9 +250,9 @@
 	      }
 	  }
       }
-    HWND hChild = LOWORD(::SendMessage(theApp->_Client,
-		           WM_MDIGETACTIVE, 0,0));	// who caused this?
-    ::SendMessage(theApp->_Client, WM_MDIDESTROY, hChild, 0);
+
+    HWND hChild = FORWARD_WM_MDIGETACTIVE(theApp->_Client, ::SendMessage); // who caused this?
+    FORWARD_WM_MDIDESTROY(theApp->_Client, hChild, ::SendMessage);
   }
 
 //======================>>> vWindow::ShowPane <<<=======================
@@ -244,11 +265,10 @@
       {
 	if (pl->pane == wpane)
 	  {
-	    (pl->pane)->ShowPane(OnOrOff);	// let each pane show hide
+	    (pl->pane)->ShowPane(OnOrOff); // let each pane show hide
 	    return;
 	  }
       }
-
     return;
   }
 
@@ -276,14 +296,14 @@
   {
     // General interface to a keyboard event
 
-    theApp->KeyIn(this, keysym, shift);	// pass on unprocessed keys to app
+    theApp->KeyIn(this, keysym, shift); // pass on unprocessed keys to app
   }
 
 //=======================>>> vWindow::KeyInEV <<<==========================
   void vWindow::KeyInEV(unsigned int keycode, int map, int extended)
   {
     // Local interface to a keyboard event
-
+  
     unsigned int WinCodes[] =
       {
 	VK_HOME, VK_LEFT, VK_UP, VK_RIGHT, VK_DOWN,
@@ -303,6 +323,8 @@
 	vk_F14, vk_F15,	vk_F16, 0
       };
 
+    ItemVal id;
+
     vKey vkey;
     int shift = 0;
 
@@ -315,7 +337,8 @@
 	vkey = keycode;
     else
       {
-	for (int ix = 0 ; WinCodes[ix] != 0 ; ++ix)
+	int ix;
+	for (ix = 0 ; WinCodes[ix] != 0 ; ++ix)
 	  {
 	    if (WinCodes[ix] == keycode)
 	      {
@@ -323,21 +346,26 @@
 		break;
 	      }
 	  }
-	if (WinCodes[ix] == 0)	 	// Ignore keys with no mapping
+	if (WinCodes[ix] == 0)	// Ignore keys with no mapping
 	    return;
       }
 
     if (vkey == vk_Tab && (shift | VKM_Shift) != 0)
 	vkey = vk_BackTab;
+    else if (vkey < ' ')
+	shift &= VKM_Shift;	// Kill VKM_Ctrl for real control chars
 
-    KeyIn(keycode,shift);			// pass on to general code
+    if (((vMenuPane*)_menuPane)->CheckAccel(vkey, shift, id))
+	MenuCommand(id);
+    else
+	KeyIn(vkey,shift);	// pass on to general code
   }
 
 //======================>>> vWindow::MenuCommand <<<==========================
   void vWindow::MenuCommand(ItemVal id)
   {
     //	Menu Commands go through here first, then the regular WindowCommand
-
+  
     WindowCommand(id, id, C_Menu);
   }
 
@@ -381,7 +409,7 @@
     top = (rct.top >= ar.top) ? rct.top - ar.top : rct.top;
     width = rct.right - rct.left;
     height = rct.bottom - rct.top;
-
+  
   }
 
 //================>>> vWindow::SetPosition <<<========================
@@ -402,7 +430,7 @@
   {
     // search all buttons in this windows to set to correct value
 
-    for (PaneList* pl = /* w_list->*/ _paneList ; pl != 0 ; pl = pl->nextPL)
+    for (PaneList* pl =	/* w_list->*/ _paneList ; pl != 0 ; pl = pl->nextPL)
       {
 	(pl->pane)->SetPaneString(id, str);
       }
@@ -434,7 +462,7 @@
 //======================>>> vWindow::WindowCommand <<<==========================
   void vWindow::WindowCommand(ItemVal id, ItemVal retval, CmdType ctype)
   {
-
+  
     SysDebug1(CmdEvents,"vWindow::WindowCommand(id: %d)\n",id)
 
     theApp->AppCommand(this, id, retval, ctype); // Pass unprocessed cmds to app
@@ -443,7 +471,7 @@
 //===================>>> vWindow::MdiWindowProc <<<=============================
   long vWindow::MdiWindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
   {
-
+  
     if (message == WM_SETFONT)
 	return 0;
     else if (message == WM_INITDIALOG)
@@ -454,59 +482,42 @@
       {
 	case WM_ACTIVATE:
 	  {
-#ifdef WIN32
-	    WORD state = LOWORD(wParam);
-	    WORD minimized = HIWORD(wParam);
-	    HWND hwnd = (HWND)lParam;
-#else
-	    WORD state = wParam;
-	    WORD minimized = LOWORD(lParam);
-	    HWND hwnd = (HWND)HIWORD(lParam);
-#endif
-//	    OnActivate(state, minimized, hwnd);
-//	      return 0;
 	    break;
 	  }
 
 	case WM_SETFOCUS:
 	  {
- //	    if (_canvasPane)
- //		_canvasPane->EnterEV();
- //	    return 0;
 	    break;
 	  }
-
-        case WM_KILLFOCUS:
+      
+	case WM_KILLFOCUS:
 	  {
-//	    if (_canvasPane)
-//		_canvasPane->LeaveEV();
-//	    return 0;
 	    break;
 	  }
-
+      
 	case WM_CREATE:
 	  {
 	    return 0;
 	  }
-
+      
 	case WM_PAINT:
 	  {
 	    if (_canvasPane)
 		_canvasPane->ExposeEV();
 	    break;
 	  }
-
-        case WM_SIZE:
+      
+	case WM_SIZE:
 	  {
 
 	    int width = LOWORD(lParam);
 	    int height = HIWORD(lParam);
 	    if (_canvasPane)
 		_canvasPane->ResizeEV(height, width);
-	    break;	// MUST also do default processing!
+	    break;			// MUST also do default processing!
 	  }
-
-	case WM_RBUTTONDOWN:		// Right mouse down
+      
+	case WM_RBUTTONDOWN:	// Right mouse down
 	  {
 	    int x = (int)LOWORD(lParam);
 	    int y = (int)HIWORD(lParam);
@@ -514,8 +525,8 @@
 		_canvasPane->MouseDown(x, y, 3);
 	    return 0;
 	  }
-
-	case WM_RBUTTONUP:		// Right mouse up
+      
+	case WM_RBUTTONUP:	// Right mouse up
 	  {
 	    int x = (int)LOWORD(lParam);
 	    int y = (int)HIWORD(lParam);
@@ -523,15 +534,15 @@
 		_canvasPane->MouseUp(x, y, 3);
 	    return 0;
 	  }
-
-        case WM_RBUTTONDBLCLK:		// Right double click
+      
+	case WM_RBUTTONDBLCLK:	// Right double click
 	  {
-//	    int x = (int)LOWORD(lParam);
-//	    int y = (int)HIWORD(lParam);
+	    //	    int x = (int)LOWORD(lParam);
+	    //	    int y = (int)HIWORD(lParam);
 	    break;
 	  }
-
-        case WM_MBUTTONDOWN:		// middle button
+      
+	case WM_MBUTTONDOWN:	// middle button
 	  {
 	    int x = (int)LOWORD(lParam);
 	    int y = (int)HIWORD(lParam);
@@ -539,24 +550,24 @@
 		_canvasPane->MouseDown(x, y, 2);
 	    return 0;
 	  }
-
-        case WM_MBUTTONUP:
+      
+	case WM_MBUTTONUP:
 	  {
 	    int x = (int)LOWORD(lParam);
 	    int y = (int)HIWORD(lParam);
 	    if (_canvasPane)
-	        _canvasPane->MouseUp(x, y, 2);
+		_canvasPane->MouseUp(x, y, 2);
 	    return 0;
 	  }
-
-        case WM_MBUTTONDBLCLK:
+      
+	case WM_MBUTTONDBLCLK:
 	  {
-//	    int x = (int)LOWORD(lParam);
-//	    int y = (int)HIWORD(lParam);
+	    //	    int x = (int)LOWORD(lParam);
+	    //	    int y = (int)HIWORD(lParam);
 	    break;
 	  }
-
-        case WM_LBUTTONDOWN:		// Left mouse down
+      
+	case WM_LBUTTONDOWN:	// Left mouse down
 	  {
 	    // We want this to work like X. The first click in an MDI window
 	    // without focus changes focus, AND sends the LBUTTONDOWN message.
@@ -566,16 +577,16 @@
 	    if (_justActivated  && _prevWin != 0)
 	      {
 		_justActivated = 0;
-                break;
+		break;
 	      }
 	    int x = (int)LOWORD(lParam);
 	    int y = (int)HIWORD(lParam);
-	     _mouseDown = 1;
+	    _mouseDown = 1;
 	    if (_canvasPane)
 		_canvasPane->MouseDown(x, y, 1);
 	    return 0;
 	  }
-
+      
 	case WM_LBUTTONUP:
 	  {
 	    int x = (int)LOWORD(lParam);
@@ -585,15 +596,15 @@
 	    _mouseDown = 0;
 	    return 0;
 	  }
-
-        case WM_LBUTTONDBLCLK:
+      
+	case WM_LBUTTONDBLCLK:
 	  {
-//	    int x = (int)LOWORD(lParam);
-//	    int y = (int)HIWORD(lParam);
+	    //	    int x = (int)LOWORD(lParam);
+	    //	    int y = (int)HIWORD(lParam);
 	    break;
 	  }
 
-        case WM_MOUSEMOVE:		// Mouse move
+	case WM_MOUSEMOVE:	// Mouse move
 	  {
 	    int x = (int)LOWORD(lParam);
 	    int y = (int)HIWORD(lParam);
@@ -609,7 +620,7 @@
 	    else if (wParam & MK_RBUTTON)
 		button = 3;
 	    // To make this work like the X version, we don't want any mouse
-            // motion unless a button is down.
+	    // motion unless a button is down.
 	    if (button < 0)
 	      {
 		if (_canvasPane)
@@ -620,17 +631,16 @@
 	    else if (_canvasPane)
 		_canvasPane->MouseMove(x, y, button);
 	    return 0;
-
 	  }
 
-        case WM_DESTROY:
+	case WM_DESTROY:
 	  {
 	    break;
 	  }
 
-        case WM_COMMAND:
+	case WM_COMMAND:
 	  {
-#ifdef WIN32
+#ifdef _WIN32
 	    WORD id = LOWORD(wParam);
 	    HWND hwnd = (HWND)lParam;
 	    WORD cmd = HIWORD(wParam);
@@ -642,83 +652,91 @@
 	    MenuCommand((ItemVal)id);
 	    return 0;
 	  }
-
+      
 	case WM_KEYDOWN:
 	  {
 	    if (wParam == VK_SHIFT || wParam == VK_CONTROL)
 		break;
 	    // Handle only keystrokes that won't generate a WM_CHAR
 	    else if ( (wParam <= VK_HELP && wParam != VK_CANCEL &&
-		       wParam != VK_ESCAPE && wParam != VK_SPACE &&
-		       wParam != VK_RETURN && wParam != VK_BACK &&
-		       wParam != VK_TAB) ||
-		      (wParam >= VK_F1 && wParam <= VK_F16)
-		    )
+		wParam != VK_ESCAPE && wParam != VK_SPACE &&
+	    wParam != VK_RETURN && wParam != VK_BACK &&
+	    wParam != VK_TAB) ||
+	    (wParam >= VK_F1 && wParam <= VK_F16) )
 	      {
-
 		KeyInEV((unsigned int)wParam, 1, 0);
 		return 0;
 	      }
 	    break;
 	  }
-
+      
 	case WM_CHAR:		// Always an ASCII character
 	  {
 	    KeyInEV((unsigned int)wParam, 0, (HIWORD(lParam) & KF_EXTENDED) != 0);
 	    return 0;
 	  }
-
+      
 	case WM_HSCROLL:
 	  {
-#ifdef WIN32
-	    WORD code = LOWORD(wParam);
-	    WORD pos = HIWORD(wParam);
-	    HWND control = (HWND)lParam;
-#else
-	    WORD code = wParam;
-	    WORD pos = LOWORD(lParam);
-	    HWND control = (HWND)HIWORD(lParam);
-#endif
-//		   break;		// @@
 	    if (_canvasPane)
-		_canvasPane->HScrollEV(code, pos, control);
+		_canvasPane->HScrollEV(GET_WM_HSCROLL_CODE(wParam, lParam),
+	    GET_WM_HSCROLL_POS(wParam, lParam),
+	    GET_WM_HSCROLL_HWND (wParam, lParam));
+
 	    return 0;
 	  }
-
-        case WM_VSCROLL:
+      
+	case WM_VSCROLL:
 	  {
-#ifdef WIN32
-	    WORD code = LOWORD(wParam);
-	    WORD pos = HIWORD(wParam);
-	    HWND control = (HWND)lParam;
-#else
-	    WORD code = wParam;
-	    WORD pos = LOWORD(lParam);
-	    HWND control = (HWND)HIWORD(lParam);
-#endif
-//       break;		// @@
+
 	    if (_canvasPane)
-		_canvasPane->VScrollEV(code, pos, control);
+		_canvasPane->VScrollEV(GET_WM_VSCROLL_CODE(wParam, lParam),
+		GET_WM_VSCROLL_POS(wParam, lParam),
+	    	GET_WM_VSCROLL_HWND (wParam, lParam));
 	    return 0;
 	  }
-
-        case WM_MDIACTIVATE:
+      
+	case WM_MDIACTIVATE:
 	  {
 	    // First, we have to change the main menu to ours
-	    if (wParam)		// we are gaining focus!
+	    int activate;
+#ifdef _WIN32
+	    /* This could be done with GET_WM_MDIACTIVATE_FACTIVATE,
+	    but it doesn't seem to be in the windows 3.1 windowsx.h */
+	    activate = lParam == (LPARAM)hWnd;
+#else
+	    activate = wParam != 0;
+#endif
+
+	    if (activate)		// we are gaining focus!
 	      {
 		if (theApp->InExit())
 		    break;
-		theApp->selectCmdPanes(this);
+
 		HMENU popup = ::GetSubMenu(_menuBar,_WindowMenuIndex); // get 1st menu item
 
+
 		if (popup)		// add the window list
-		    ::SendMessage(theApp->_Client, WM_MDISETMENU,0,
-		                MAKELONG(_menuBar,popup));
+		  {
+#ifdef _WIN32
+		    BOOL refresh = 1;
+#else
+		    BOOL refresh = 0;
+#endif
+		    FORWARD_WM_MDISETMENU(theApp->_Client, refresh,
+		    (HMENU) _menuBar, popup, ::SendMessage);
+		  }
+
 		_prevWin = _curWin;
 		_curWin = hWnd;
 		_justActivated = 1;
-		::DrawMenuBar(theApp->_Frame);	// Make it change
+		::DrawMenuBar(theApp->_Frame); // Make menu change
+		theApp->selectCmdPanes(this);  // Select our cmd bars
+		// now resize to be sure they are drawn. This only matters
+		// when the different child windows have different menus
+                // or menus set on or off
+                FORWARD_vWM_ResizeMDIClient(theApp->_Frame,SendMessage);
+
 		if (_canvasPane)
 		  {
 		    if (_canvasPane->_paneType == P_TextCanvas)
@@ -730,13 +748,20 @@
 		    _canvasPane->EnterEV();
 		  }
 	      }
-	    else		// we are losing focus
+	    else			// we are losing focus
 	      {
 		HMENU appframe = theApp->AppFrameMenu();
 		HMENU popup = GetSubMenu(appframe,0);
+#ifdef _WIN32
+		BOOL refresh = 1;
+#else
+		BOOL refresh = 0;
+#endif
+		FORWARD_WM_MDISETMENU(theApp->_Client, refresh,
+		  appframe, popup, ::SendMessage);  // Change to our menu
+
+		theApp->selectCmdPanes(this);  // Need to select command bars
 
-		::SendMessage(theApp->_Client, WM_MDISETMENU,0,
-		            MAKELONG(appframe,popup));
 		if (_canvasPane)
 		  {
 		    _canvasPane->LeaveEV();
@@ -745,11 +770,12 @@
 			::DestroyCaret();
 		      }
 		  }
+
 	      }
 
 	    return 0;
 	  }
-
+      
 	case WM_DROPFILES:
 	  {
 	    break;
@@ -760,7 +786,7 @@
 	    CloseWin();
 	    return 0;
 	  }
-
+      
 	case WM_CLOSE:
 	  {
 	    CloseWin();
