@@ -15,7 +15,7 @@
 //                                                                         //
 // vapp.cxx - The vApp control object (for X11R5)                          //
 //                                                                         //
-// Copyright (C) 1995,1996  Bruce E. Wampler                                    //
+// Copyright (C) 1995,1996  Bruce E. Wampler                               //
 //                                                                         //
 // This file is part of the V C++ GUI Framework.                           //
 //                                                                         //
@@ -39,6 +39,8 @@
 #include <v/vapp.h>		// my header file
 #include <v/vwindow.h>		// Win header
 #include <v/vcmdwin.h>
+#include <v/vctlclrs.h>
+#include <v/vcolor.h>
 
 //#include <X11/StringDefs.h>
 
@@ -50,12 +52,42 @@
     vApp* theApp = NULL;	// to be filled in upon instantiation
     DebugMask DebugState;
 
+    // Define the default color scheme - shades of gray
+    // #define UseBlueColors to get a Motif-like look for the default
+
+#ifdef UseBlueColors
+
+#define vLightGray "LightSteelBlue2"	// status bar background (204, 80% gray)
+#define vMedGray "LightSteelBlue"	// dialog, command bar background (191,75%)
+#define vDarkGray "LightSteelBlue3"	// Menu bar, button faces (179,70%)
+#define vDarkShadow "LightSteelBlue4"	// dark shadows for buttons
+#define vLightShadow "LightSteelBlue1"	// light shadows for buttons (222,87%)
+
+#else		// Default V Gray scheme
+
+#define vLightGray "#CCCCCCCCCCCC"	// status bar background (204, 80% gray)
+#define vMedGray "#BFBFBFBFBFBF"	// dialog, command bar background (191,75%)
+#define vDarkGray "#B3B3B3B3B3B3"	// Menu bar, button faces (179,70%)
+#define vDarkShadow "#7F7F7F7F7F7F"	// dark shadows for buttons (127,50%)
+#define vLightShadow "#DEDEDEDEDEDE"	// light shadows for buttons (222,87%)
+
+#endif
+
     // define the structures we need to recover the XtDefaultFont info
 
     typedef struct
      { 
 	Pixel copy_fg;
 	Pixel copy_bg;
+
+	Pixel dialogBG;
+	Pixel statusBarBG;
+	Pixel menuBarBG;
+	Pixel controlBG;
+	Pixel controlFace;
+	Pixel lightControlShadow;
+	Pixel darkControlShadow;
+
 	XFontStruct *font;
      } AppData;
 
@@ -81,6 +113,73 @@
 	XtRString,
 	XtDefaultBackground
       },
+
+//--------------------------------------------
+      {
+	"vDialogBG",
+	XtCForeground,
+	XtRPixel,
+	sizeof(Pixel),
+	XtOffsetOf(AppData,dialogBG),
+	XtRString,
+	vMedGray
+      },
+      {
+	"vStatusBarBG",
+	XtCForeground,
+	XtRPixel,
+	sizeof(Pixel),
+	XtOffsetOf(AppData,statusBarBG),
+	XtRString,
+	vLightGray
+      },
+      {
+	"vMenuBarBG",
+	XtCForeground,
+	XtRPixel,
+	sizeof(Pixel),
+	XtOffsetOf(AppData,menuBarBG),
+	XtRString,
+	vDarkGray
+      },
+      {
+	"vControlBG",
+	XtCForeground,
+	XtRPixel,
+	sizeof(Pixel),
+	XtOffsetOf(AppData,controlBG),
+	XtRString,
+	vDarkGray
+      },
+      {
+	"vControlFace",
+	XtCForeground,
+	XtRPixel,
+	sizeof(Pixel),
+	XtOffsetOf(AppData,controlFace),
+	XtRString,
+	vMedGray
+      },
+      {
+	"vLightControlShadow",
+	XtCForeground,
+	XtRPixel,
+	sizeof(Pixel),
+	XtOffsetOf(AppData,lightControlShadow),
+	XtRString,
+	vLightShadow
+      },
+      {
+	"vDarkControlShadow",
+	XtCForeground,
+	XtRPixel,
+	sizeof(Pixel),
+	XtOffsetOf(AppData,darkControlShadow),
+	XtRString,
+	vDarkShadow
+      },
+//--------------------------------------------
+
       {
 	XtNfont,
 	XtCFont,
@@ -92,13 +191,26 @@
       },
     };
 
+   // Define three shades of gray used to draw dialogs and controls.
+   // LightGray is for the status bar background
+   // Med Gray is for the dialog and command bar background
+   // DarkGray is for the menu bar and button faces
+
+
+  Pixel _vStatusBarBG;
+  Pixel _vDialogBG;
+  Pixel _vMenuBarBG;
+  Pixel _vControlBG;
+  Pixel _vControlFace;
+  Pixel _vLightControlShadow;
+  Pixel _vDarkControlShadow;
 
   static char copyright[] =
     "****> Copyright (C) 1995 Bruce E. Wampler; under terms of the\
     GNU Library General Public License, version 2 <****";
 
 //========================>>> vApp::vApp <<<=======================
-  vApp::vApp(char* appName) : vBaseItem(appName)	// constructor
+  vApp::vApp(char* appName, int simSDI) : vBaseItem(appName)	// constructor
   {
     // First, set the global pointer to the main App. This happens
     // when the user declares the instance of the app, either from
@@ -113,6 +225,7 @@
     _display = NULL;
     _appContext = NULL;
     _WindowList = NULL;		// no windows registered
+    _simSDI = simSDI;		// not used in X
 
     _DefaultHeight = 150;	// default sizes for canvas window
     _DefaultWidth = 350;
@@ -222,7 +335,7 @@
 
     _vHandle = XtAppInitialize(
 	&_appContext,		// The Xt App Context
-	_name,			// A name
+	_name,		// my name
 	Options, XtNumber(Options),
 	&local_argc, argv,	// XtAppInitialize eats it args (e.g., -font)
 	NULL, NULL, 0);
@@ -241,6 +354,7 @@
     // placement of other windows later)
 
     _display = XtDisplay(_vHandle);	// Get the display
+    _xwindow = 0;
 
     _DisplayHeight = DisplayHeight(_display,0);
     _DisplayWidth = DisplayWidth(_display,0);
@@ -281,51 +395,44 @@
 
     if (_depth <= 1)
       {
-	_lightGray = _medGray = _darkGray =    // white, not gray
+	_vStatusBarBG =
+	_vDialogBG =
+	_vMenuBarBG =
+	_vControlBG =
+	_vControlFace =
+	_vLightControlShadow =
 	_backgroundPix = XWhitePixel(theApp->display(),0);
+	_vDarkControlShadow =
 	_foregroundPix = XBlackPixel(theApp->display(),0);
       }
     else
       {
-	_lightGray = GetGrayPixel(225);		// three shades of gray
-	_medGray = GetGrayPixel(200);
-	_darkGray = GetGrayPixel(175);
-
 	_backgroundPix = app_data.copy_bg;
 	_foregroundPix = app_data.copy_fg;
+
+	// Get the control colors
+	_vDialogBG = app_data.dialogBG;
+	_vStatusBarBG = app_data.statusBarBG;
+	_vMenuBarBG = app_data.menuBarBG;
+	_vControlBG = app_data.controlBG;
+	_vControlFace = app_data.controlFace;
+	_vLightControlShadow = app_data.lightControlShadow;
+	_vDarkControlShadow = app_data.darkControlShadow;
+      }
+
+    for (int icx = 0 ; icx < 16 ; ++icx)
+      {
+	(void) vStdColors[icx].pixel();	// Force all colors to map
       }
 
     _running = 1;
 
   }
 
-//========================>>> vApp::GetGrayPixel <<<=======================
-  Pixel vApp::GetGrayPixel(int Shade)
+//========================>>> vApp::xWindow <<<=======================
+  Window vApp::xWindow()
   {
-    // Get a shade of gray
-
-    XColor xc;
-    Pixel GPixel;
-
-    // Convert from 0-255 scale of V to 0-65535 scale of X
-    xc.red =  
-    xc.green = 
-    xc.blue = Shade * 256;
-    xc.flags = DoRed | DoBlue | DoGreen;
-    xc.pixel = 0;
-
-    // Now, map the RGB to a Pixmap index (if color!)
-
-    if (Xdepth() <= 1 ||
-	!XAllocColor(display(), Xcolormap(), &xc))
-      {
-	GPixel = XWhitePixel(theApp->display(),0);
-      }
-    else
-      {
-	GPixel = xc.pixel;
-      }      
-    return GPixel;
+    return (_xwindow == 0 ? (_xwindow = XtWindow(_vHandle)) : _xwindow);
   }
 
 //========================>>> vApp::NewAppWin <<<=======================
@@ -367,11 +474,12 @@
     // events to get through.
     XEvent an_event;
 
-    if (XtAppPending(theApp->appContext()) == 0)
-	return;
-
-    XtAppNextEvent(theApp->appContext(),&an_event); // get events
-    XtDispatchEvent(&an_event);             // and dispatch them 
+    // Clean up event queue of events we care about.
+    while (XtAppPending(theApp->appContext()) & XtIMXEvent)
+      {
+	XtAppNextEvent(theApp->appContext(),&an_event); // get events
+	XtDispatchEvent(&an_event);             // and dispatch them 
+      }
   }
 
 //======================>>> vApp::Exit <<<=======================
@@ -485,12 +593,13 @@
 //========================>>> vApp::unregisterWindow <<<=======================
   void vApp::unregisterWindow(vWindow *Win)
   {
-    WindList *curWin, *tmp, *last;
+    WindList *curWin, *tmp, *last, *next;
 
     last = 0;
 
-    for (curWin = _WindowList ; curWin !=0 ; curWin = curWin->nextWinList)
+    for (curWin = _WindowList ; curWin !=0 ; curWin = next)
       {
+	next = curWin->nextWinList;
 	if (curWin->window == Win)
 	  {
 
