@@ -10,11 +10,22 @@
 //===============================================================
 
 #include <v/vcanvas.h>		// our header
+#include <v/vctlclrs.h>
 
 extern "C"
 {
 #include <X11/Xaw/Paned.h>
+
+#ifdef USE3D
+#include <v/awscl3d.h>
+#define sbWidgetClass scrollbar3dWidgetClass
+#define sbThickness 15
+#else
 #include <X11/Xaw/Scrollbar.h>
+#define sbWidgetClass scrollbarWidgetClass
+#define sbThickness 10
+#endif
+
 #include <v/canvas.h>		// our own canvas widget
 }
 
@@ -105,6 +116,7 @@ extern "C"
 	canvasWidgetClass,	// widget class 
 	_baseW,			// parent widget
 	XtNshowGrip, FALSE,
+	XtNallowResize,1,
 	XtNheight,(_parentWin->_WinHeight)+16,
 	XtNwidth,(_parentWin->_WinWidth)+16,
 	XtNhSpace, 1,
@@ -153,23 +165,31 @@ extern "C"
 
     _drawVScroll = XtVaCreateManagedWidget(
 	"vCanvasVScroll",				// name
-	scrollbarWidgetClass,		// class
+	sbWidgetClass,			// class
 	_drawCanvas,			// parent
 	XtNscrollProc, VscrollProcList,	// callback for scrolling
 	XtNjumpProc, VjumpProcList,	// callback for scrolling
 	XtNheight, _parentWin->_WinHeight,
-	XtNthickness, 10,		// scrollbar param
+	XtNthickness, sbThickness,	// scrollbar param
+#ifdef USE3D
+	XtNbackground,_vControlFace,
+	XtNscrollbar3dBackground, _vControlBG,
+#endif
 	NULL);
 
     _drawHScroll = XtVaCreateManagedWidget(
 	"vCanvasHScroll",
-	scrollbarWidgetClass,
+	sbWidgetClass,
 	_drawCanvas,
 	XtNscrollProc, HscrollProcList,	// callback for scrolling
 	XtNjumpProc, HjumpProcList,	// callback for scrolling
 	XtNorientation,XtorientHorizontal,	// scrollbar param
-	XtNthickness, 10,		// scrollbar param
+	XtNthickness, sbThickness,	// scrollbar param
 	XtNwidth, _parentWin->_WinWidth,
+#ifdef USE3D
+	XtNbackground,_vControlFace,
+	XtNscrollbar3dBackground, _vControlBG,
+#endif
 	NULL);
 
     XtAddCallback(_drawCanvas, XtNcallback, CchangeCB, (XtPointer)this);
@@ -191,6 +211,7 @@ extern "C"
     _parentWin->_WinHeight = _height = curH;	// update the local stuff
     _parentWin->_WinWidth = _width = curW;
 
+    _Drawable = 0;
   }
 
 //==================>>> vCanvasPane::CreateDC <<<========================
@@ -210,6 +231,10 @@ extern "C"
 	XtNheight, &curH,
 	NULL);
 
+    if (curW > 0x7FFF)			// supposed to be unsigned, but
+	curW = 0;			// can get confused
+    if (curH > 0x7FFF)			// supposed to be unsigned, but
+	curH = 0;			// can get confused
     _parentWin->_WinHeight = _height = curH;
     _parentWin->_WinWidth = _width = curW;
 
@@ -230,6 +255,22 @@ extern "C"
       }
   }
 
+//=====================>>> vCanvasPane::SetHeightWidth <<<============================
+  void vCanvasPane::SetHeightWidth(int height, int width)
+  {
+    Dimension curW = width;
+    Dimension curH = height;
+
+    _parentWin->_WinHeight = _height = curH;
+    _parentWin->_WinWidth = _width = curW;
+
+    XtVaSetValues(_drawCanvas,	// get values for the draw canvas
+	XtNwidth, curW,		// its new h and w
+	XtNheight, curH,
+	NULL);
+
+  }
+
 //------------------------- cursor Stuff ---------------------------------
 //=====================>>> vCanvasPane::SetCursor <<<============================
   void vCanvasPane::SetCursor(VCursor id)
@@ -403,24 +444,24 @@ extern "C"
   {
     Arg args[3];		// Used to pass float value
 
-    float fshown = (float)(Shown/100.);
-    float ftop = (float)(Top/100.);		// work with local copies
-
-    // This was taken from an example on how to set float values
-    // portably.
-
-    if (sizeof(float) > sizeof(XtArgVal))
+    union
       {
-	XtSetArg(args[0], XtNtopOfThumb, &ftop);
-	XtSetArg(args[1], XtNshown, &fshown);
+	XtArgVal arg_value;
+	float float_value;
+      } shown_value, top_value;
+
+    shown_value.float_value = float (Shown / 100.0);
+    top_value.float_value = float (Top / 100.0);
+
+    if (sizeof (float) > sizeof (XtArgVal))
+      {
+	XtSetArg(args[0], XtNtopOfThumb, &top_value);
+	XtSetArg(args[1], XtNshown, &shown_value);
       }
     else
       {
-	XtArgVal* l_top = (XtArgVal *) &ftop;
-	XtArgVal* l_shown = (XtArgVal *) &fshown;
-
-	XtSetArg(args[0], XtNtopOfThumb, *l_top);
-	XtSetArg(args[1], XtNshown, *l_shown);
+	XtSetArg(args[0], XtNtopOfThumb, top_value.arg_value);
+	XtSetArg(args[1], XtNshown, shown_value.arg_value);
       }
 
     XtSetValues(_drawHScroll, args, 2);		// change it!
@@ -494,24 +535,24 @@ extern "C"
   {
     Arg args[3];		// Used to pass float value
 
-    float fshown = (float)Shown / 100.;
-    float ftop = (float)(Top / 100.);		// work with local copies
-
-    // This was taken from an example on how to set float values
-    // portably.
-
-    if (sizeof(float) > sizeof(XtArgVal))
+    union
       {
-	XtSetArg(args[0], XtNtopOfThumb, &ftop);
-	XtSetArg(args[1], XtNshown, &fshown);
+	XtArgVal arg_value;
+	float float_value;
+      } shown_value, top_value;
+
+    shown_value.float_value = float (Shown / 100.0);
+    top_value.float_value = float (Top / 100.0);
+
+    if (sizeof (float) > sizeof (XtArgVal))
+      {
+	XtSetArg(args[0], XtNtopOfThumb, &top_value);
+	XtSetArg(args[1], XtNshown, &shown_value);
       }
     else
       {
-	XtArgVal* l_top = (XtArgVal *) &ftop;
-	XtArgVal* l_shown = (XtArgVal *) &fshown;
-
-	XtSetArg(args[0], XtNtopOfThumb, *l_top);
-	XtSetArg(args[1], XtNshown, *l_shown);
+	XtSetArg(args[0], XtNtopOfThumb, top_value.arg_value);
+	XtSetArg(args[1], XtNshown, shown_value.arg_value);
       }
 
     XtSetValues(_drawVScroll, args, 2);		// change it!
