@@ -10,10 +10,7 @@
 //===============================================================
 
 
-extern "C"
-{
 #include <math.h>
-}
 
 #include <v/vpsdc.h>
 
@@ -369,6 +366,50 @@ ellipsedict /mtrx matrix put\n\
     // no op
   }
 
+//==================>>> vPostScriptDC::DrawColorPoints <<<======================
+  void vPostScriptDC::DrawColorPoints(int x, int y, int nPoints, vColor* pointList)
+  {
+    // Draw a bunch of color points. 
+
+    char hexval[4];
+
+    if (nPoints < 1)
+	return;
+
+    int xx = Scale(x+_tx);
+    int yy = Scale(y+_ty);
+
+    unsigned char bval;
+	
+    // This code adapted from 24 bit icon code
+
+    *pstream << "\ngsave 32 dict begin\n/pix24 "
+	    << nPoints * 3 << " string def\n"
+	    << xx << " " << yy << " translate "
+	    << nPoints << " " << 1 << " scale\n"
+	    << nPoints << " " << 1 << " 8 ["
+	    << nPoints << " 0 0 " << 1 << " 0 0]\n"
+	    << "{currentfile pix24 readhexstring pop} false 3 colorimage\n";
+
+    for (int c8 = 0 ; c8 < nPoints ; c8++)
+      {
+	bval = pointList[c8].r();	// red
+	ByteToStr(bval,hexval);
+	*pstream << hexval;
+
+	bval = pointList[c8].g();	// green
+	ByteToStr(bval,hexval);
+	*pstream << hexval;
+
+	bval = pointList[c8].b();	// blue
+	ByteToStr(bval,hexval);
+	*pstream << hexval;
+      }
+
+    *pstream << "\nend grestore\n";
+
+  }
+
 //====================>>> vPostScriptDC::DrawEllipse <<<==========================
   void vPostScriptDC::DrawEllipse(int x, int y, int width, int height)
   {
@@ -576,6 +617,35 @@ ellipsedict /mtrx matrix put\n\
 		<< xe << " " << ye << " lineto stroke\n";
   }
 
+//====================>>> vPostScriptDC::DrawLines <<<==========================
+  void vPostScriptDC::DrawLines(vLine* lineList)
+  {
+    // Draw a line from x,y to xend,yend
+
+    if (!pstream)
+	return;
+
+    SetGCtoPen();
+    
+    for (vLine* lp = lineList ; lp ; ++lp)
+      {
+	int xx = Scale(lp->x+_tx);
+	int yy = Scale(lp->y+_ty);
+	int xe = Scale(lp->xend+_tx);
+	int ye = Scale(lp->yend+_ty);
+
+
+	if (xx < 0) xx = 0;
+	if (yy < 0) yy = 0;
+	if (xe < 0) xe = 0;
+	if (ye < 0) ye = 0;		// X hates negative coords
+
+        *pstream << "newpath "
+		<< xx << " " << yy << " moveto "
+		<< xe << " " << ye << " lineto stroke\n";
+      }
+  }
+
 //==================>>> vPostScriptDC::DrawPoint <<<======================
   void vPostScriptDC::DrawPoint(int x, int y)
   {
@@ -596,6 +666,29 @@ ellipsedict /mtrx matrix put\n\
 
   }
 
+//==================>>> vPostScriptDC::DrawPoints <<<======================
+  void vPostScriptDC::DrawPoints(vPoint* pointList)
+  {
+
+    if (!pstream)
+	return;
+
+    SetGCtoPen();
+    for (vPoint* pl = pointList ; pl ; ++pl)
+      {
+	int xx = Scale(pl->x+_tx);
+	int yy = Scale(pl->y+_ty);
+
+	if (xx < 0) xx = 0;
+	if (yy < 0) yy = 0;
+
+	*pstream 	<< "newpath\n"
+		<< xx << " " << yy << " moveto "
+		<< xx+1 << " " << yy << " lineto\nstroke\n";
+      }
+
+  }
+
 //==================>>> vPostScriptDC::DrawPolygon <<<======================
   void vPostScriptDC::DrawPolygon(int n, vPoint points[], int fillStyle)
   {
@@ -705,6 +798,61 @@ ellipsedict /mtrx matrix put\n\
       }
   }
 
+//==================>>> vPostScriptDC::DrawRectangles <<<======================
+  void vPostScriptDC::DrawRectangles(vRect* rectList)
+  {
+
+    if (!pstream)
+	return;
+
+    if (_pen.penStyle != vTransparent)
+	SetGCtoPen();
+    if (_brush.brushStyle != vTransparent)
+	SetGCtoBrush();
+
+    for (vRect* rp ; rp ; ++rp)
+      {
+	int xx = Scale(rp->x+_tx);
+	int yy = Scale(rp->y+_ty);
+	int w = Scale(rp->w);
+	int h = Scale(rp->h);
+
+	if (h == 0 || w == 0)	// ignore 0 h/w
+	    continue;
+
+	if (w < 0)		// convert to something we like
+	  {
+	    w = -w;		// make a positive width
+	    xx = xx - w;	// translate x
+	  }
+	if (h < 0)
+	  {
+	    h = -h;		// positive h
+	    yy = yy - h;	// translate x
+	  }
+
+	if (_brush.brushStyle != vTransparent)
+	  {
+	    *pstream << "newpath\n"
+	    << xx << " " << yy << " moveto\n"
+	    << (xx+w) << " " << yy << " lineto\n"
+	    << (xx+w) << " " << (yy+h) << " lineto\n"
+	    << xx << " " << (yy+h) << " lineto\n"
+	    << "closepath fill\n";
+	  }
+
+	if (_pen.penStyle != vTransparent)
+	  {
+	    *pstream << "newpath\n"
+	    << xx << " " << yy << " moveto\n"
+	    << (xx+w) << " " << yy << " lineto\n"
+	    << (xx+w) << " " << (yy+h) << " lineto\n"
+	    << xx << " " << (yy+h) << " lineto\n"
+	    << "closepath stroke\n";
+	  }
+      }
+  }
+
 //================>>> vPostScriptDC::DrawRoundedRectangle <<<===================
   void vPostScriptDC::DrawRoundedRectangle(int x, int y,
 	int width, int height, int radius)
@@ -868,7 +1016,7 @@ ellipsedict /mtrx matrix put\n\
     // need to check the current font and change to our font if
     // it is not the same
 
-    if (!pstream)
+    if (!pstream || !text || !*text)
 	return;
 
     int xx = Scale(x+_tx);
@@ -959,7 +1107,9 @@ ellipsedict /mtrx matrix put\n\
 
     // Must check color first since PS "brush and pen" share color
 
-    SetPSColor(_brush.GetColor());		// set up color
+    
+    vColor bc = _brush.GetColor();
+    SetPSColor(bc);		// set up color
 
     if (_curBrush == _brush)			// no-op if no change
 	return;
@@ -1008,12 +1158,12 @@ ellipsedict /mtrx matrix put\n\
 //=====================>>> vPostScriptDC::SetGCtoPen <<<========================
   void vPostScriptDC::SetGCtoPen()
   {
-    SetPSColor(_pen.GetColor());
 
     if (_curPen == _pen)			// no-op if no change
 	return;
 
-    SetPSColor(_pen.GetColor());
+    vColor pc = _pen.GetColor();
+    SetPSColor(pc);
 
     if (_curPen.GetWidth() != _pen.GetWidth())	// change width
       {
